diff --git a/xgm.py b/xgm.py
index 42104c04de812cded7102ec3584fd90576604cf0..5f8b0d12fc98b18d8d57197afd411bcd6d9042a8 100644
--- a/xgm.py
+++ b/xgm.py
@@ -392,8 +392,9 @@ def mcpPeaks(data, intstart, intstop, bkgstart, bkgstop, mcp=1, t_offset=None, n
             bkgstart: trace index of background start
             bkgstop: trace index of background stop
             mcp: MCP channel number
             t_offset: index separation between two pulses. If None, checks the
                 pulse pattern and determine the t_offset assuming mininum pulse 
                 separation of 220 ns and digitizer resolution of 2 GHz.
             npulses: number of pulses. If None, takes the maximum number of
                 pulses according to the bunch patter (field 'npulses_sase3')
@@ -407,8 +408,8 @@ def mcpPeaks(data, intstart, intstop, bkgstart, bkgstop, mcp=1, t_offset=None, n
         raise ValueError("Source not found: {}!".format(keyraw))
     if npulses is None:
         npulses = int(data['npulses_sase3'].max().values)
-    sa3 = data['sase3'].where(data['sase3']>1)
     if t_offset is None:
+        sa3 = data['sase3'].where(data['sase3']>1)
         if npulses > 1:
             #Calculate the number of pulses between two lasing pulses (step)
             step = sa3.where(data['npulses_sase3']>1, drop=True)[0,:2].values
@@ -417,7 +418,7 @@ def mcpPeaks(data, intstart, intstop, bkgstart, bkgstop, mcp=1, t_offset=None, n
             t_offset = 440 * step
         else:
             t_offset = 1
-    results = xr.DataArray(np.empty((sa3.shape[0], npulses)), coords=sa3.coords,
+    results = xr.DataArray(np.empty((data.trainId.shape[0], npulses)), coords=data[keyraw].coords,
                            dims=['trainId', 'MCP{}fromRaw'.format(mcp)])
     for i in range(npulses):
         a = intstart + t_offset*i
@@ -430,11 +431,15 @@ def mcpPeaks(data, intstart, intstop, bkgstart, bkgstop, mcp=1, t_offset=None, n
 
 
 def getTIMapd(data, mcp=1, use_apd=True, intstart=None, intstop=None,
-              bkgstart=None, bkgstop=None, t_offset=None, npulses=None):
+              bkgstart=None, bkgstop=None, t_offset=None, npulses=None, 
+              stride=1):
     ''' Extract peak-integrated data from TIM where pulses are from SASE3 only.
         If use_apd is False it calculates integration from raw traces. 
         The missing values, in case of change of number of pulses, are filled
         with NaNs.
+        If no bunch pattern info is available, the function assumes that
+        SASE 3 comes first and that the number of pulses is fixed in both
+        SASE 1 and 3.
         
         Inputs:
             data: xarray Dataset containing MCP raw traces (e.g. 'MCP1raw')
@@ -444,12 +449,25 @@ def getTIMapd(data, mcp=1, use_apd=True, intstart=None, intstop=None,
             bkgstop: trace index of background stop
             t_offset: index separation between two pulses
             mcp: MCP channel number
-            npulses: number of pulses to compute
-
+            npulses: int, optional. Number of pulses to compute. Needed if
+                no bunch pattern info is available.
+            stride: int, optional. Used to select pulses in the APD array if
+                no bunch pattern info is available.
         Output:
             tim: DataArray of shape trainId only for SASE3 pulses x N 
                  with N=max(number of pulses per train)
     '''
+    if 'sase3' not in data:
+        print('Missing bunch pattern info!\n'
+              +'Retrieving {} SASE 3 pulses assuming that '.format(npulses)
+              +'SASE 3 pulses come first.')
+        if use_apd:
+            tim = data['MCP{}apd'.format(mcp)][:,:npulses:stride]
+        else:
+            tim = mcpPeaks(data, intstart, intstop, bkgstart, bkgstop, mcp=mcp, 
+                       t_offset=t_offset, npulses=npulses)
+        return tim
+    
     sa3 = data['sase3'].where(data['sase3']>1, drop=True)
     npulses_sa3 = data['npulses_sase3']
     maxpulses = int(npulses_sa3.max().values)
@@ -392,8 +392,9 @@ def mcpPeaks(data, intstart, intstop, bkgstart, bkgstop, mcp=1, t_offset=None, n
             bkgstop: trace index of background stop
             mcp: MCP channel number
-            t_offset: index separation between two pulses. If None, checks the
-                pulse pattern and determine the t_offset assuming mininum pulse 
+            t_offset: index separation between two pulses. Needed if bunch
+                pattern info is not available. If None, checks the pulse
+                pattern and determine the t_offset assuming mininum pulse 
                 separation of 220 ns and digitizer resolution of 2 GHz.
             npulses: number of pulses. If None, takes the maximum number of